/*##############################################################################

    HPCC SYSTEMS software Copyright (C) 2012 HPCC Systems.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
############################################################################## */

#ifndef __HTTPPROT_HPP_
#define __HTTPPROT_HPP_

#ifndef esp_http_decl
    #define esp_http_decl
#endif

//Jlib
#include "jliball.hpp"

//SCM Interfaces
#include "esp.hpp"
//#include "IAEsp.hpp"

//SCM Core
#include "espthread.hpp"

#include "espprotocol.hpp"
#include "http/platform/httpbinding.hpp"
#include "securesocket.hpp"

//STL
#include <algorithm>
#include <string>
#include <map>
using namespace std;

class CPooledHttpThread : public CInterface, implements IPooledThread
{
private:
    Owned<ISocket> m_socket;
    CEspApplicationPort* m_apport;
    StringAttr m_context;
    int m_MaxRequestEntityLength;
    bool m_is_ssl;
    ISecureSocketContext* m_ssctx;

public:
    IMPLEMENT_IINTERFACE;

    virtual ~CPooledHttpThread();
    virtual void init(void *param);
    virtual void main();
    virtual bool stop()
    {
        return true;
    }
    virtual bool canReuse() { return true; }

    virtual void setMaxRequestEntityLength(int len) {m_MaxRequestEntityLength = len;}
    virtual int getMaxRequestEntityLength() { return m_MaxRequestEntityLength; }
};

class CHttpThreadPoolFactory : public CInterface, implements IThreadFactory
{
public:
    IMPLEMENT_IINTERFACE;

    virtual IPooledThread *createNew()
    {
        return new CPooledHttpThread();
    }
};


class CHttpThread: public CEspProtocolThread
{
private:
    CEspApplicationPort* m_apport;
    bool m_viewConfig;
    StringAttr m_context;
    int m_MaxRequestEntityLength;
    CHttpThread(bool viewConfig);
    CHttpThread(ISocket *sock, bool viewConfig);
    bool m_is_ssl;
    ISecureSocketContext* m_ssctx;

public:
    IMPLEMENT_IINTERFACE;

    CHttpThread(ISocket *sock, CEspApplicationPort* apport, bool viewConfig, bool isSSL = false, ISecureSocketContext* ssctx = NULL);
    
    virtual ~CHttpThread();
    virtual bool onRequest();

    virtual void setMaxRequestEntityLength(int len) {m_MaxRequestEntityLength = len;}
    virtual int getMaxRequestEntityLength() { return m_MaxRequestEntityLength; }
};


class esp_http_decl CHttpProtocol : public CEspProtocol
{
private:
    int m_maxConcurrentThreads;
    int m_threadCreateTimeout;
public:
    CHttpProtocol();
    virtual ~CHttpProtocol();

    virtual void Link(void) const
    {
        CEspProtocol::Link(); 
    }
    
    virtual bool Release(void) const
    {
        return CEspProtocol::Release();
    }

    virtual void init(IPropertyTree * cfg, const char * process, const char * protocol);

    virtual bool notifySelected(ISocket *sock,unsigned selected);
//IEspProtocol
    virtual const char * getProtocolName();
};

class esp_http_decl CSecureHttpProtocol : public CEspProtocol
{
private:
    StringBuffer m_certfile;
    StringBuffer m_privkeyfile;
    StringBuffer m_passphrase;
    Owned<ISecureSocketContext> m_ssctx;

    Owned<IPropertyTree> m_config;
    int m_maxConcurrentThreads;

public:
    CSecureHttpProtocol(IPropertyTree *cfg);
    virtual ~CSecureHttpProtocol();

    virtual void init(IPropertyTree * cfg, const char * process, const char * protocol);

    virtual bool notifySelected(ISocket *sock,unsigned selected);
//IEspProtocol
    virtual const char * getProtocolName();
};

#endif //__HTTPPROT_HPP_

